'use strict';

var react = require('react');

function parseDefinition(propValue, propDefinition, prefix = "") {
    if (typeof propDefinition === 'string') { // prop: string
        if (propValue === true) {
            return propDefinition.split(" ").map(s => prefix + s); // split string into multiple parts because we always return array of single classes
        }
    }
    else if (propDefinition instanceof Array) { //  prop: string[]
        if (propValue === true) {
            return propDefinition.map(d => parseDefinition(true, d, prefix)).flat();
        }
    }
    else if (typeof propDefinition === 'function') {
        return parseDefinition(true, propDefinition(propValue), prefix);
    }
    else if (propDefinition.toString() === "[object Object]") { //  prop: {key: value?}
        // test if propValue exists in propDefinition
        const maybeObj = propDefinition === null || propDefinition === void 0 ? void 0 : propDefinition[("" + propValue)];
        if (maybeObj) {
            return parseDefinition(true, maybeObj, prefix);
        }
        // prop: {pseudo: value}
        return Object.keys(propDefinition).map(key => {
            return parseDefinition(propValue, propDefinition[key], prefix + `${key}:`);
        }).flat();
    }
    return [];
}

const shouldMix = (when, optionArr) => {
    return when.every(w => {
        if (typeof w !== "string") {
            return false;
        }
        if (w.endsWith("*")) { // allow wildcards
            return optionArr.some(o => o.split(".").shift() === w.split(".").shift());
        }
        return optionArr.some(o => o === w); // simnple find
    });
};
const mixFunction = (when, run) => ({ when, run });
const mixAddClass = (when, arg) => {
    return {
        when,
        run: (css) => {
            const argCss = parseDefinition(true, arg);
            argCss.forEach(s => css.add(s));
        }
    };
};
const mixRemoveClass = (when, arg) => {
    return {
        when,
        run: (css) => {
            const argCss = parseDefinition(true, arg);
            argCss.forEach(s => css.delete(s));
        }
    };
};

/**
 * Hook that parses a set of parameters based on a configuration object and returns applicable classnames
 *
 *
 * @export
 * @template P extends ComponentAttributes
 * @template EL extends ExtendableElement = "div"
 * @param {ComposerConfig<P, EL>} config the configuration object
 * @param {CombinedProps<P, EL>} props the props to parse
 * @returns {ClassComposerReturns<A>} { className, forwardProps }
 */
function useClassComposer(config, props) {
    return react.useMemo(() => {
        // useMemo does nothing here, this should be an external hook with memoized return values
        const propOptions = [];
        const propCss = new Set();
        const optionEntries = Object.entries(config.options);
        const optionAlias = Object.entries(config.alias || []);
        const forwardProps = {};
        // load base
        parseDefinition(true, config.base).forEach(s => propCss.add(s));
        // load props
        for (const [key, propValue] of Object.entries(props)) {
            const entry = optionEntries
                .find(([k]) => {
                return k === key
                    || k === "$" + key
                    || k === "$$" + key
                    || optionAlias.find(([alias, target]) => alias === key && target === k);
            }); // find key or alias
            if (entry) {
                const [entryKey, entryValue] = entry;
                if (propValue && entryValue) {
                    parseDefinition(entryKey.startsWith("$$") ? true : propValue, entryValue).forEach(s => propCss.add(s));
                    propOptions.push(`${key}.${propValue}`);
                }
                if (entryKey.startsWith("$")) { // forward props starting with $
                    Object.assign(forwardProps, { [key]: propValue });
                }
            }
            else {
                // save prop to forward
                propOptions.push(`${key}.${propValue}`);
                Object.assign(forwardProps, { [key]: propValue });
            }
        }
        for (const mixer of config.mix || []) {
            if (shouldMix(mixer.when, propOptions)) {
                mixer.run(propCss);
            }
        }
        if (props.className) {
            props.className.split(" ").forEach((s) => propCss.add(s));
        }
        // merge with overwrite classname
        const className = Array.from(propCss).join(" ");
        return {
            forwardProps,
            className,
        };
    }, [props]);
}

/**
 * Create a React component that extends and creates a forwardRef of native HTML element.
 *
 *
 * @template P extends CustomAttributes
 * @template EL extends ExtendableElement
 * @param {EL} el defaults to "div"
 * @param {ComposerConfig<P, ComponentProps<EL>>} config
 * @param {Partial<P>} [defaults={}]
 * @returns {ComposedComponent<P, EL>}
 *
 * @see {ComposerConfig}
 * @see {ClassDefinition}
 */
function createComponent(el, config, defaults = {}) {
    return react.forwardRef((innerProps, ref) => {
        const { className, forwardProps } = useClassComposer(config, Object.assign(Object.assign({}, defaults), innerProps));
        return react.createElement(el, Object.assign(Object.assign({ ref }, forwardProps), { className }), innerProps.children);
    });
}

exports.createComponent = createComponent;
exports.mixAddClass = mixAddClass;
exports.mixFunction = mixFunction;
exports.mixRemoveClass = mixRemoveClass;
exports.parseDefinition = parseDefinition;
exports.shouldMix = shouldMix;
exports.useClassComposer = useClassComposer;
//# sourceMappingURL=index.js.map
